package common;

import java.io.IOException;
import java.io.InputStream;
import java.net.Socket;

public class TCPPortCommunicator extends PortCommunicator {

	private Socket socket;
	private boolean defunct;
	private TransmissionUnit transWaiting;

	public TCPPortCommunicator(Socket socket) {
		super("TCP comm " + socket.getLocalSocketAddress());
		this.socket = socket;
	}
	
	public boolean isDefunct() {
		return defunct;
	}

	@Override
	public void run() {
		byte [] data = new byte [Common.DEFAULT_MTU];
		int dataLen;
		try {
			InputStream sockIn = socket.getInputStream();
			while (!defunct) {
				dataLen = sockIn.read(data, 0, data.length);
				if (this.isInterrupted()) {
					break;
				}
				if (dataLen == -1) {
					dataLen = data.length;
				}
				Log.get().println(Log.LEVEL_SPAM, "Local network TCP data came in.");
				
				byte [] dataCopy = new byte[dataLen];
				System.arraycopy(data, 0, dataCopy, 0, dataLen);
				TransmissionUnit newUnit = new TransmissionUnit(dataCopy, true);
				
				synchronized (this) {
					transWaiting = newUnit;
					this.wait();
				}
				
				firePacketReceived();
			}
		} catch (Exception e) {
			if (defunct) {
				//it was on purpose
			} else {
				Log.get().println(Log.LEVEL_ERROR, "TCP port communicator receiver errored.");
				Log.get().exception(Log.LEVEL_ERROR, e);
			}
		}

		defunct = true;
		try {
			socket.close();
		} catch (Exception e) {}
		Log.get().println(Log.LEVEL_INFO, "Local TCP socket closed.");
	}

	@Override
	public void sendPacket(byte[] data) {
		if (isDefunct()) {
			//TODO do something
			return;
		}

		try {
			socket.getOutputStream().write(data, 0, data.length);
		} catch (IOException e) {
			if (defunct) {
				//we did it on purpose
			} else {
				Log.get().println(Log.LEVEL_ERROR, "TCP port communicator sender errored.");
				Log.get().exception(Log.LEVEL_ERROR, e);
				defunct = true;
			}
		}
	}

	@Override
	public void close() {
		defunct = true;
		try {
			socket.close();
		} catch (IOException e) {
		}
	}

	@Override
	public boolean shouldBeClosed() {
		return transWaiting == null && isDefunct();
	}

	@Override
	public synchronized TransmissionUnit receiveIncoming() {
		TransmissionUnit tu = transWaiting;
		transWaiting = null;
		
		if (tu != null) {
			this.notifyAll();
		}
		
		return tu;
	}
	

}
